unit ftd2xxdyn;

interface

uses SysUtils,Windows;

Type FT_Result = Integer;

type FT_Device_Info_Node = record
  Flags         : DWord;
  DeviceType    : Dword;
  ID            : DWord;
  LocID         : DWord;
  SerialNumber  : array [0..15] of Char;
  Description   : array [0..63] of Char;
  DeviceHandle  : DWord;
end;

type TDWordptr = ^DWord;

type FT_Handle = ^DWord;


Const
// FT_Result Values
    FT_OK = 0;
    FT_INVALID_HANDLE = 1;
    FT_DEVICE_NOT_FOUND = 2;
    FT_DEVICE_NOT_OPENED = 3;
    FT_IO_ERROR = 4;
    FT_INSUFFICIENT_RESOURCES = 5;
    FT_INVALID_PARAMETER = 6;
    FT_SUCCESS = FT_OK;
    FT_INVALID_BAUD_RATE = 7;
    FT_DEVICE_NOT_OPENED_FOR_ERASE = 8;
    FT_DEVICE_NOT_OPENED_FOR_WRITE = 9;
    FT_FAILED_TO_WRITE_DEVICE = 10;
    FT_EEPROM_READ_FAILED = 11;
    FT_EEPROM_WRITE_FAILED = 12;
    FT_EEPROM_ERASE_FAILED = 13;
    FT_EEPROM_NOT_PRESENT = 14;
    FT_EEPROM_NOT_PROGRAMMED = 15;
    FT_INVALID_ARGS = 16;
    FT_OTHER_ERROR = 17;
// FT_Open_Ex Flags
    FT_OPEN_BY_SERIAL_NUMBER = 1;
    FT_OPEN_BY_DESCRIPTION = 2;
    FT_OPEN_BY_LOCATION = 4;
// FT_List_Devices Flags
    FT_LIST_NUMBER_ONLY = $80000000;
    FT_LIST_BY_INDEX = $40000000;
    FT_LIST_ALL = $20000000;
// Baud Rate Selection
    FT_BAUD_300 = 300;
    FT_BAUD_600 = 600;
    FT_BAUD_1200 = 1200;
    FT_BAUD_2400 = 2400;
    FT_BAUD_4800 = 4800;
    FT_BAUD_9600 = 9600;
    FT_BAUD_14400 = 14400;
    FT_BAUD_19200 = 19200;
    FT_BAUD_38400 = 38400;
    FT_BAUD_57600 = 57600;
    FT_BAUD_115200 = 115200;
    FT_BAUD_230400 = 230400;
    FT_BAUD_460800 = 460800;
    FT_BAUD_921600 = 921600;
// Data Bits Selection
    FT_DATA_BITS_7 = 7;
    FT_DATA_BITS_8 = 8;
// Stop Bits Selection
    FT_STOP_BITS_1 = 0;
    FT_STOP_BITS_2 = 2;
// Parity Selection
    FT_PARITY_NONE = 0;
    FT_PARITY_ODD = 1;
    FT_PARITY_EVEN = 2;
    FT_PARITY_MARK = 3;
    FT_PARITY_SPACE = 4;
// Flow Control Selection
    FT_FLOW_NONE = $0000;
    FT_FLOW_RTS_CTS = $0100;
    FT_FLOW_DTR_DSR = $0200;
    FT_FLOW_XON_XOFF = $0400;
// Purge Commands
    FT_PURGE_RX = 1;
    FT_PURGE_TX = 2;
// Notification Events
    FT_EVENT_RXCHAR = 1;
    FT_EVENT_MODEM_STATUS = 2;
// Modem Status
    CTS = $10;
    DSR = $20;
    RI = $40;
    DCD = $80;


// IO Buffer Sizes
    FT_In_Buffer_Size = $10000;    // 64k
    FT_In_Buffer_Index = FT_In_Buffer_Size - 1;
    FT_Out_Buffer_Size = $10000;    // 64k
    FT_Out_Buffer_Index = FT_Out_Buffer_Size - 1;
// DLL Name
    FT_DLL_Name = 'FTD2XX.DLL';

//Classic functions
function FT_Read(ftHandle:Dword; FTInBuf:Pointer; BufferSize:LongInt; ResultPtr:Pointer):FT_Result;
function FT_Write(ftHandle:Dword; FTOutBuf:Pointer; BufferSize:LongInt; ResultPtr:Pointer):FT_Result;

function FT_OpenEx(pvArg1:Pointer; dwFlags:Dword; ftHandle:FT_HANDLE):FT_Result;
function FT_ResetDevice(ftHandle:Dword):FT_Result;
function FT_SetTimeouts(ftHandle:Dword; ReadTimeout,WriteTimeout:Dword):FT_Result;
function FT_Purge(ftHandle:Dword; Mask:Dword):FT_Result;
function FT_SetUSBParameters(ftHandle:Dword; InSize,OutSize:Dword):FT_Result;

function FT_Close(ftHandle:Dword):FT_Result;

//Dynamic linking libraries
function FT_ConnectLibrary:boolean;
function FT_DisconnectLibrary:boolean;

implementation

var DLLHandle:THandle;

function FT_ConnectLibrary:boolean;
begin
  DLLHandle := 0;
  DLLHandle := LoadLibrary(FT_DLL_Name);
  if DLLHandle=0 then
   Result:=false
  else
   Result:=true;
end;

function FT_DisconnectLibrary:boolean;
begin
  if DLLHandle<>0 then
   begin
     FreeLibrary(DLLHandle);
     DLLHandle:=0;
     Result:=true;
   end
  else
   Result:=false;
end;



function FT_Read(ftHandle:Dword; FTInBuf:Pointer; BufferSize:LongInt; ResultPtr:Pointer):FT_Result;
var Func:function (ftHandle:Dword; FTInBuf:Pointer; BufferSize:LongInt; ResultPtr:Pointer):FT_Result stdcall;
begin
 Result:=FT_INVALID_HANDLE;
 if DLLHandle>0 then
  begin
   @Func:=GetProcAddress(DLLHandle,'FT_Read');
   if @Func<>nil then
    Result:=Func(ftHandle,FTInBuf,BufferSize,ResultPtr);
  end;
end;


function FT_Write(ftHandle:Dword; FTOutBuf:Pointer; BufferSize:LongInt; ResultPtr:Pointer):FT_Result;
var Func:function(ftHandle:Dword; FTOutBuf:Pointer; BufferSize:LongInt; ResultPtr:Pointer):FT_Result stdcall;
begin
 Result:=FT_INVALID_HANDLE;
 if DLLHandle>0 then
  begin
   @Func:=GetProcAddress(DLLHandle,'FT_Write');
   if @Func<>nil then
    Result:=Func(ftHandle,ftOutBuf,BufferSize,ResultPtr);
  end;
end;


function FT_OpenEx(pvArg1:Pointer; dwFlags:Dword; ftHandle:FT_HANDLE):FT_Result;
var Func:function(pvArg1:Pointer; dwFlags:Dword; ftHandle:FT_HANDLE):FT_Result stdcall;
begin
 Result:=FT_INVALID_HANDLE;
 if DLLHandle>0 then
  begin
   @Func:=GetProcAddress(DLLHandle,'FT_OpenEx');
   if @Func<>nil then
    Result:=Func(pvArg1,dwFlags,ftHandle);
  end;
end;


function FT_ResetDevice(ftHandle:Dword):FT_Result;
var Func:function(ftHandle:Dword):FT_Result stdcall;
begin
 Result:=FT_INVALID_HANDLE;
 if DLLHandle>0 then
  begin
   @Func:=GetProcAddress(DLLHandle,'FT_ResetDevice');
   if @Func<>nil then
    Result:=Func(ftHandle);
  end;
end;


function FT_SetTimeouts(ftHandle:Dword; ReadTimeout,WriteTimeout:Dword):FT_Result;
var Func:function(ftHandle:Dword; ReadTimeout,WriteTimeout:Dword):FT_Result stdcall;
begin
 Result:=FT_INVALID_HANDLE;
 if DLLHandle>0 then
  begin
   @Func:=GetProcAddress(DLLHandle,'FT_SetTimeouts');
   if @Func<>nil then
    Result:=Func(ftHandle,ReadTimeout,WriteTimeout);
  end;
end;

function FT_Purge(ftHandle:Dword; Mask:Dword):FT_Result;
var Func:function(ftHandle:Dword; Mask:Dword):FT_Result stdcall;
begin
 Result:=FT_INVALID_HANDLE;
 if DLLHandle>0 then
  begin
   @Func:=GetProcAddress(DLLHandle,'FT_Purge');
   if @Func<>nil then
    Result:=Func(ftHandle,Mask);
  end;
end;


function FT_SetUSBParameters(ftHandle:Dword; InSize,OutSize:Dword):FT_Result;
var Func:function(ftHandle:Dword; InSize,OutSize:Dword):FT_Result stdcall;
begin
 Result:=FT_INVALID_HANDLE;
 if DLLHandle>0 then
  begin
   @Func:=GetProcAddress(DLLHandle,'FT_SetUSBParameters');
   if @Func<>nil then
    Result:=Func(ftHandle,InSize,OutSize);
  end;
end;


function FT_Close(ftHandle:Dword):FT_Result;
var Func:function(ftHandle:Dword):FT_Result stdcall;
begin
 Result:=FT_INVALID_HANDLE;
 if DLLHandle>0 then
  begin
   @Func:=GetProcAddress(DLLHandle,'FT_Close');
   if @Func<>nil then
    Result:=Func(ftHandle);
  end;
end;



end.
