unit usbgecko;

interface

uses Windows,SysUtils,Classes,Graphics;

{$DEFINE DYNAMIC}

type TWiiStatus=(wsRunning,wsPaused,wsBreakpoint,wsLoader,wsUnknown);

type TWiiLanguage=(wlNoOverride,wlJapanese,wlEnglish,wlGerman,wlFrench,
 wlSpanish,wlItalian,wlDutch,wlChineseSimplified,wlChineseTraditional,
 wlKorean);
type TWiiPatches=(wpNoPatches,wpPAL60,wpVIDTV,wpPAL60VIDTV,wpNTSC,wpNTSCVIDTV);

type EUSBGeckoException=class(Exception)
end;

type ECheatStreamException=class(Exception)
end;

type ERegStreamException=class(Exception)
end;

type TOnChunkTransfer=procedure(current,finl,bytestrans,memrange:integer;lastfail:boolean) of object;

type TUSBGecko=class
       private
    FConnected:boolean;
   FOnNewChunk:TOnChunkTransfer;
     protected
     FFTHandle:DWord;
      function gecko_writebyte(lpBuffer:Pointer;dwBytesToWrite:DWORD;lpdwBytesWritten:LPDWORD):integer;
      function gecko_readbyte(lpBuffer:Pointer;dwBytesToRead:DWORD;lpdwBytesReturned:LPDWORD):integer;
     procedure Breakpoint(address:Cardinal;add:Byte;aligned:boolean);
     procedure FixDumps;
      function InitGecko:boolean;
      procedure SendChunkUpdate(current,finl,bytestrans,memrange:integer;lastfail:boolean);
        public
      function VersionRequest:Byte;
      function Connect:boolean;
      function WiiStatus:TWiiStatus;
     procedure Disconnect;
      function RareCommand(id:byte):integer;
      function Reconnect:boolean;
     procedure Dump(startdump,enddump:Cardinal;saveStream:TStream);
     procedure Poke(address,value:Cardinal);
     procedure Poke16(address:Cardinal;extvalue:Word);
     procedure Poke8(address:Cardinal;extvalue:Byte);
     procedure Upload(address,length:Cardinal;sendStream:TStream);
     procedure SendRegs(sendStream:TStream);
     procedure SendCheats(sendStream:TStream);
     procedure Pause;
     procedure Resume;
     procedure Hook(pause:boolean=false;language:TWiiLanguage=wlNoOverride;patch:TWiiPatches=wpNoPatches);
     procedure Step;
      function Peek(address:Cardinal):Cardinal;
     procedure CancelBreakpoints;
     procedure BreakpointR(address:Cardinal;aligned:boolean=false);
     procedure BreakpointW(address:Cardinal;aligned:boolean=false);
     procedure BreakpointRW(address:Cardinal;aligned:boolean=false);
     procedure BreakpointX(address:Cardinal);
      function BreakpointHit:boolean;
     procedure GetRegisters(strm:TStream);
     procedure SendFail;
     procedure Screenshot(output:TBitmap);
   constructor Create;
    destructor Destroy;override;
      property Connected:boolean read FConnected;
      property onNewChunk:TOnChunkTransfer read FOnNewChunk write FOnNewChunk;
end;

//General byte swap functions for
function BSwap(Input:Cardinal):Cardinal; //32 bit integers
function BSwap16(Input:Word):Word;       //16 bit integers
function BSwap64(Input:int64):int64;     //64 bit integers

implementation

{$IFDEF DYNAMIC}
uses ftd2xxdyn; //dynamic connection to the ftdi chip driver
{$ELSE}
uses ftd2xx;    //static connection to the ftdi chip driver
{$ENDIF}

//Command list
const  cmd_poke08 : Byte = $01;
       cmd_poke16 : Byte = $02;
      cmd_pokemem : Byte = $03;
      cmd_readmem : Byte = $04;
        cmd_pause : Byte = $06;
     cmd_unfreeze : Byte = $07;
   cmd_breakpoint : Byte = $09;
  cmd_breakpointx : Byte = $10;
     cmd_sendregs : Byte = $2F;
      cmd_getregs : Byte = $30;
     cmd_cancelbp : Byte = $38;
   cmd_sendcheats : Byte = $40;
       cmd_upload : Byte = $41;
         cmd_hook : Byte = $42;
    cmd_hookpause : Byte = $43;
         cmd_step : Byte = $44;
       cmd_status : Byte = $50;   
  cmd_nbreakpoint : Byte = $89;
      cmd_version : Byte = $99;

//Reply list
            GCACK : Byte = $AA;
          GCRETRY : Byte = $BB;
           GCFAIL : Byte = $CC;
           GCDONE : Byte = $FF;
          GCBPHit : Byte = $11;

//New gecko version identifier
         GCNewVer : Byte = $80;

//package size
           packetsize = $F800;
        uplpacketsize = $F80;

//possible gecko version
type TGeckoVersions=set of Byte;

//the only allowed version so far is GCNewVer
const GCAllowedVersions:TGeckoVersions=[$80];

{ TUSBGecko }

//16 bit byteswap - high speed assembler
function BSwap16(Input:Word):Word;
asm
  xchg al,ah
end;

//64 bit byteswap - high speed assembler
function BSwap64(Input:int64):int64;
asm
 mov edx,dword ptr [Input]
 mov eax,dword ptr [Input+4]
 bswap edx
 bswap eax
end;

//32 bit byteswap -  high speed assembler
function BSwap(Input:Cardinal):Cardinal;
asm
  bswap eax;
end;

//lower an address value to an address good by 4 bytes
function Lower(Input:Cardinal):Cardinal;
//begin
//  Result:=Input DIV 4*4;
//end;
asm
  and eax,$FFFFFFFC
end;

//lower an address value to an address good by 8 bytes
function Lower8(Input:Cardinal):Cardinal;
//begin
//  Result:=Input DIV 8*8;
//end;
asm
  and eax,$FFFFFFF8
end;

(*
 Function returns boolean=true once a breakpoint has hit, otherwise false.
 *)
function TUSBGecko.BreakpointHit:boolean;
var   returnvalue:integer;
           RxSent:DWord;
             data:Byte;
begin
 //read one byte
 returnvalue := gecko_readbyte(@data, 1, @RxSent);
 //did reading work?
 if(returnvalue<>1) then
  Result:=false;

 //is it a bphit signal
 Result:=data=GCBPHit;
end;

 (*
   Tries to retrieve the USB Gecko version - it must return
   one of the allwed values (like 0x80)
  *)
function TUSBGecko.VersionRequest:Byte;
var   returnvalue:integer;
    TxSent,RxSent:DWord;
          retries:integer;
              ver:Byte;
begin
 //Clear buffers
 InitGecko;

 //Send version request
 returnvalue := gecko_writebyte(@cmd_version, 1, @TxSent);
 if(returnvalue<>1) then
  raise EUSBGeckoException.Create('Error sending command');

 //3 attempts to wait for a reply
 retries:=0;

 Result:=0;

 repeat
  returnvalue := gecko_readbyte(@ver, 1, @RxSent);
  if(returnvalue=1) then
  if ver in GCAllowedVersions then
   begin
    //Version valid
    Result:=ver;
    break;
   end;
  Inc(retries);
 until retries=3;
end;

(*
   Data breakpoints. (private function for good reason)
   Input: Address to check
   add: 5 for read breakpoint, 6 for write, 7 for readwrite
   aligned=true: breakpoint only good to 8 bytes
   aligned=false: breakpoint exact!
 *)
procedure TUSBGecko.Breakpoint(address: Cardinal;add:Byte;aligned:boolean);
var   returnvalue:integer;
          orgaddy:Cardinal;
    TxSent,RxSent:DWord;
          retries:integer;
              ver:Byte;
              new:boolean;
              fbp:Int64;
begin
 orgaddy:=BSwap(address); //original address in bigendian
 address:=BSwap(Lower8(address)+add); //breakpoint data in bigendian (lowered address + add)
 new:=false; //are we using an old gecko or a new?

 if not aligned then //unaligned breakpoints do not require the new gecko anyway
  begin
   new:=VersionRequest>=GCNewVer; //is it a new one?
  end;

 if not new then //old breakpoint method
  begin
   //Send breakpoint request
   returnvalue := gecko_writebyte(@cmd_breakpoint, 1, @TxSent);
   if(returnvalue<>1) then
    raise EUSBGeckoException.Create('Error sending command');

   //Send address
   returnvalue := gecko_writebyte(@address, 4, @TxSent);
   if(returnvalue<>1) then
    raise EUSBGeckoException.Create('Error sending command');
  end
 else //new breakpoint
  begin
   //Send command
   returnvalue := gecko_writebyte(@cmd_nbreakpoint, 1, @TxSent);
   if(returnvalue<>1) then
    raise EUSBGeckoException.Create('Error sending command');

   //Send address in both old form and the original address
   //old form identifies breakpoint type
   fbp:=address+Int64(Int64(orgaddy) shl 32);

   returnvalue := gecko_writebyte(@fbp, 8, @TxSent);
   if(returnvalue<>1) then
    raise EUSBGeckoException.Create('Error sending command');
  end;
end;

(*
 Read brekpoint:
  aligned=true: good to 8 bytes
  aligned=false: exact!
 *)
procedure TUSBGecko.BreakpointR(address: Cardinal;aligned:boolean=false);
begin
 BreakPoint(Address,5,aligned);
end;

(*
 Write brekpoint:
  aligned=true: good to 8 bytes
  aligned=false: exact!
 *)
procedure TUSBGecko.BreakpointRW(address: Cardinal;aligned:boolean=false);
begin
 BreakPoint(Address,7,aligned);
end;

(*
 Read/Write brekpoint:
  aligned=true: good to 8 bytes
  aligned=false: exact!
 *)
procedure TUSBGecko.BreakpointW(address: Cardinal;aligned:boolean=false);
begin
 BreakPoint(Address,6,aligned);
end;

(*
 execute breakpoint
 always exact up to 4 bytes!
*)
procedure TUSBGecko.BreakpointX(address: Cardinal);
var   returnvalue:integer;
           TxSent:DWord;
begin
 //clear stack
 InitGecko;

 //Big endian address (+3 = execute bp identifier)
 address:=BSwap(Lower(address)+3);

 //Send command
 returnvalue := gecko_writebyte(@cmd_breakpointx, 1, @TxSent);
 if(returnvalue<>1) then
  raise EUSBGeckoException.Create('Error sending command');

 //Send address
 returnvalue := gecko_writebyte(@address, 4, @TxSent);
 if(returnvalue<>1) then
  raise EUSBGeckoException.Create('Error sending command');
end;

(* Cancels breakpoint *)
procedure TUSBGecko.CancelBreakpoints;
begin
 //Send cancelbp
 RareCommand(cmd_cancelbp);
end;

(* Resets Gecko - required before some commands to check stability -
 returns true in case of success *)
function TUSBGecko.InitGecko:boolean;
var status:FT_Result;
begin
 Result:=false;
 // Reset the Device
 status := FT_ResetDevice(FFTHandle);
 if(status <> FT_OK) then
  begin
   FT_Close(FFTHandle);
   raise EUSBGeckoException.Create(Format('Error: Couldnt Reset Device %d',[status]));
  end;
 // Clear RX buffer
 status := FT_Purge(FFTHandle,FT_PURGE_RX);
 if(status <> FT_OK) then
  begin
 	 FT_Close(FFTHandle);
 	 raise EUSBGeckoException.Create(Format('Error: Problem clearing buffers %d',[status]));
  end;
 // Clear TX buffer
 status := FT_Purge(FFTHandle,FT_PURGE_TX);
 if(status <> FT_OK) then
  begin
   FT_Close(FFTHandle);
	 raise EUSBGeckoException.Create(Format('Error: Problem clearing buffers %d',[status]));
  end;
 Result:=true;
end;

(*
 Connect to USB Gecko
 Returns true if everything goes fine!
*)
function TUSBGecko.Connect: boolean;
var status:FT_Result;
    name:Pointer;
begin
 //IF connected - then disconnect first!
 if FConnected then
  Disconnect;
 FConnected:=false;
 Result:=false;

 //in case of dynamic linking the library first has to be loaded
 {$IFDEF DYNAMIC}
 //Open device
 if not FT_ConnectLibrary then
  raise EUSBGeckoException.Create('Error: Couldn''t connect to USB Gecko. Please check Installation');
 {$ENDIF}

 // Open by Serial Number
 name:=PChar('GECKUSB0');
 status := FT_OpenEx(name, FT_OPEN_BY_SERIAL_NUMBER, @FFTHandle);
 if(status <> FT_OK) then
  raise EUSBGeckoException.Create('Error: Couldn''t connect to USB Gecko. Please check Installation');

 // Set Timeouts to 2 seconds
 status := FT_SetTimeouts(FFTHandle,2000,2000);	// 2 Second Timeout
 if(status <> FT_OK) then
  begin
   FT_Close(FFTHandle);
   raise EUSBGeckoException.Create(Format('Error: Timeouts failed to set %d',[status]));
  end;
 // Set packet size (not ftdi packet size) 64K is for USB 2.0 d2xx, may need to lower
 status := FT_SetUSBParameters(FFTHandle,65536,0);
 if(status <> FT_OK) then
  begin
   FT_Close(FFTHandle);
	 raise EUSBGeckoException.Create(Format('Error: Couldnt Set USB Parameters %d\n',[status]));
  end;

 //Reset Gecko
 if InitGecko then
  begin
   // Required
   Sleep(150);

   FConnected:=true;

   Result:=true;
  end;
end;

(*
 Constructor.. simply creates the interface
*)
constructor TUSBGecko.Create;
begin
  inherited;
 FConnected:=false;
end;

(*
 Destructor.. simply creates the interface
*)
destructor TUSBGecko.Destroy;
begin
 if FConnected then
  Disconnect;

  inherited;
end;

(*
 Disconnection function
*)
procedure TUSBGecko.Disconnect;
begin
 if FConnected then
   begin
    //Close Handle of Gecko
    FT_Close(FFTHandle);
    //in case of dynamic linking, also unload the library
    {$IFDEF DYNAMIC}
    //Close device
    if not FT_DisconnectLibrary then
     raise EUSBGeckoException.Create('Error: Couldn''t disconnect from USB Gecko. Please check Installation');
    {$ENDIF}
   end;

 //We're out!!
 FConnected:=FALSE;
end;

(*
 Now a real core function
 Dumping memory into a stream
 statdump = Start Address
 enddump = End address
 saveStream = Output Stream
*)
procedure TUSBGecko.Dump(startdump, enddump: Cardinal; saveStream: TStream);
var GeckoMemRange:Int64;
        memlength:Cardinal;
    TxSent,RxSent:DWord;
      returnvalue:integer;
     responseback:Byte;

            chunk:Integer;
       fullchunks:Integer;
        allchunks:Integer;
        lastchunk:Integer;

            retry:integer;
           buffer:string;
begin
 //Reset connection
 InitGecko;

 //How many bytes of data to I need to transfer
 memlength:=enddump-startdump;

 //How many chunks do I need to split this data into
 //How big is the last chunk?
 fullchunks:=memlength DIV packetsize;
  lastchunk:=memlength MOD packetsize;

 //How many chunks do I need to transfer?
 allchunks:=fullchunks;
 if lastchunk=0 then
  Inc(allchunks);

 //OLD!!!!
 //if lastchunk=0 then
 // FixDumps;

 //The Gecko wants us to send it in big endian form
 //[start address 32 bit] [end address 32 bit]
 GeckoMemRange:=BSwap64(Int64(startdump) shl 32+enddump);

 //Send command
 returnvalue:=gecko_writebyte(@cmd_readmem,1,@TxSent);
 if returnvalue<>1 then
  raise EUSBGeckoException.Create('Unable to send dump command to USBGecko!');

 //Check reply.. I want a GCACK command
 retry:=0;
 while(retry<10) do
  begin
   returnvalue:=gecko_readbyte(@responseback,1,@RxSent);
   if returnvalue<>1 then
    raise EUSBGeckoException.Create('Return value '+IntToStr(returnvalue)+' while asking for status!');
   if responseback=GCACK then
    break;
   if retry=9 then
    raise EUSBGeckoException.Create('Invalid status reply - reply is not GCACK - last reply was: '+IntToHex(responseback,2));
   Inc(retry);
  end;

 //Now we send what data we want
 returnvalue:=gecko_writebyte(@GeckoMemRange,8,@TxSent);
 if returnvalue<>1 then
  raise EUSBGeckoException.Create('Failed sending memrange!');

 //We start with chunk 0
 chunk:=0;
 retry:=0;

 while(chunk < fullchunks) do
  begin
    //Send an update to the progress bar - if one is implemented!
    SendChunkUpdate(chunk,allchunks,chunk*packetsize,memlength,retry<>0);
    SetLength(buffer,packetsize);
    //Retrieve package
    returnvalue:=gecko_readbyte(@buffer[1],packetsize,@RxSent);
    if returnvalue=2 then
     begin
       Inc(retry);
       if retry=3 then
        begin
          //Give up too many retries!
          gecko_writebyte(@GCFAIL,1,@TxSent);
          raise EUSBGeckoException.Create('Too many retries while trying to receive a package!');
        end;
       //Retry this package
       gecko_writebyte(@GCRETRY,1,@TxSent);
       continue;
     end
    else if returnvalue=0 then
     begin
       //Major fail, give up
       gecko_writebyte(@GCFAIL,1,@TxSent);
       raise EUSBGeckoException.Create('Gecko error - please check your USB Gecko connection!');
     end;
    //write received package to output stream
    saveStream.Write(buffer[1],packetsize);
    //reset retry counter
    retry:=0;
    //increase chunk counter
    Inc(chunk);
    //everything went fine, acknowledge package!
    gecko_writebyte(@GCACK,1,@TxSent);
  end;

 //is there a final package we have to transmit?
 while(lastchunk>0) do
  begin
   //blablabla, read above!
    SendChunkUpdate(fullchunks,allchunks,fullchunks*packetsize,memlength,retry<>0);
    SetLength(buffer,lastchunk);
    returnvalue:=gecko_readbyte(@buffer[1],lastchunk,@RxSent);
    if returnvalue=2 then
     begin
       Inc(retry);
       if retry=3 then
        begin
          //Give up too many retries!
          gecko_writebyte(@GCFAIL,1,@TxSent);
          raise EUSBGeckoException.Create('Too many retries while trying to receive a package!');
        end;
       gecko_writebyte(@GCRETRY,1,@TxSent);
       continue;
     end
    else if returnvalue=0 then
     begin
       //Major fail, give up
       gecko_writebyte(@GCFAIL,1,@TxSent);
       raise EUSBGeckoException.Create('Gecko error - please check your USB Gecko connection!');
     end;
    //if not jff then
     saveStream.Write(buffer[1],lastchunk);
    retry:=0;
    lastchunk:=0;
    gecko_writebyte(@GCACK,1,@TxSent);
  end;
 SetLength(buffer,0);
end;

(* Function deactivated but kept in case it might be needed again *)
procedure TUSBGecko.FixDumps;
const F800Fix:array[0..7] of Cardinal=($356BFFFF,$4082FFC4,$3C808000,$60842738,
 $80640000,$2C030000,$41820008,$4BFFFE49);
var Data:TMemoryStream;
     i,v:integer;
begin
(* if Peek($80001D34)<>$60000000 then
  Poke($80001D34,$60000000);

 if Peek($80001E68)=$41820050 then
  begin
   Data:=TMemoryStream.Create;
   for i := 0 to 7 do
    begin
     v:=BSwap(F800Fix[i]);
     Data.Write(v,4);
    end;
   Data.Position:=0;
   Upload($80001EAC,Data.Size,Data);
   FreeAndNil(Data);

   Poke($80001E68,$4182004C);
  end;*)
end;

(*
 C conversion of the read_byte code
 *)
function TUSBGecko.gecko_readbyte(lpBuffer: Pointer; dwBytesToRead: DWORD;
  lpdwBytesReturned: LPDWORD): integer;
var status:FT_Result;
begin
 status := FT_Read(FFTHandle, lpBuffer, dwBytesToRead, lpdwBytesReturned);
 if (status = FT_OK) then
  begin
	 //Packet is not right, return 2, could use as packet retry code
	 if(lpdwBytesReturned^ <> dwBytesToRead) then
     begin
   		Result:=2;
      exit;
     end;
	 end
	else
   begin
		// Fatal, unplugged cable or your mum cut the power :)
		Result:=0;
		//Disconnect;
		//raise EUSBGeckoException.Create('Error: Read Error. Closing\n');
	 end;
 Result:=1;
end;

(*
 C conversion of the write_byte code
 *)
function TUSBGecko.gecko_writebyte(lpBuffer: Pointer; dwBytesToWrite: DWORD;
  lpdwBytesWritten: LPDWORD): integer;
var status:integer;
begin
 status := FT_Write(FFTHandle, lpBuffer, dwBytesToWrite, lpdwBytesWritten);
 if (status = FT_OK) then
  begin
  // Packet is not right, return 2, could use as packet retry code
	 if (lpdwBytesWritten^ <> dwBytesToWrite) then
    begin
		 Result:=2;
     exit;
     end;
   end
	else
   begin
		//Disconnect;
		Result:=0;
		//raise EUSBGeckoException.Create('Error: Write Error. Closing.\n');
   end;
 Result:=1;
end;

(*
 Receive registers in breakpoint case and store them to a stream
 *)
procedure TUSBGecko.GetRegisters(strm:TStream);
var returnvalue:Cardinal;
  TxSent,RxSent:DWord;
       buffer:string;
       cmd:Word;
       new:boolean;
       ver,retries:Byte;
       blength:Word;
begin
 //We're using a new Gecko version?
 new:=VersionRequest>=GCNewVer;

 //If yes, then we also receive the floating point registers
 //The package we receive is therefore larger
 if new then blength:=$120
 else blength:=$A0;

 //Send commands
 returnvalue := gecko_writebyte(@cmd_getregs, 1, @TxSent);
 if(returnvalue<>1) then
  raise EUSBGeckoException.Create('Error sending command');

 (*returnvalue := gecko_writebyte(@cmd_pause, 1, @TxSent);
 if(returnvalue<>1) then
  raise EUSBGeckoException.Create('Error sending command');*)

 //Read registers
 SetLength(buffer,blength);
 returnvalue := gecko_readbyte(@buffer[1], blength, @RxSent);
 if(returnvalue <> 1) then
  begin
   //Disconnect;
   raise EUSBGeckoException.Create('Error: Couldn''t get response from console');
  end;

 //Strore them to the stream
 strm.Write(buffer[1],blength);
end;

(* Hook function
   pause=true: game starts paused
   pause=false: games immediately starts
   language: set language
   patches: apply patches
*)
procedure TUSBGecko.Hook(pause:boolean=false;language:TWiiLanguage=wlNoOverride;patch:TWiiPatches=wpNoPatches);
var returnvalue:Cardinal;
         TxSent:DWord;
            cmd:Byte;
begin
 //Send hookpause or hook
 if pause then
  returnvalue := gecko_writebyte(@cmd_hookpause, 1, @TxSent)
 else
  returnvalue := gecko_writebyte(@cmd_hook, 1, @TxSent);

 if(returnvalue<>1) then
  raise EUSBGeckoException.Create('Error sending command');

 //Apply the language
 if language<>wlNoOverride then
  begin
   cmd:=Ord(language)-1;
   returnvalue := gecko_writebyte(@cmd, 1, @TxSent);
   if(returnvalue<>1) then
    raise EUSBGeckoException.Create('Error sending command');
  end
 else
  begin
   cmd:=$CD;
   returnvalue := gecko_writebyte(@cmd, 1, @TxSent);
   if(returnvalue<>1) then
    raise EUSBGeckoException.Create('Error sending command');
  end;

 //Apply the patch
 cmd:=Ord(patch);
 returnvalue := gecko_writebyte(@cmd, 1, @TxSent);
 if(returnvalue<>1) then
  raise EUSBGeckoException.Create('Error sending command');
end;

//Pause command
procedure TUSBGecko.Pause;
var returnvalue:Cardinal;
       TxSent:DWord;
begin
 returnvalue := gecko_writebyte(@cmd_pause, 1, @TxSent);
 if(returnvalue<>1) then
  raise EUSBGeckoException.Create('Error initiating transfer');
end;


//Peek a 32 bit value - technically just dump 32 bits and output them
//as little endian integer
function TUSBGecko.Peek(address: Cardinal): Cardinal;
var MemStrm:TMemoryStream;
        buf:Cardinal;
        old:TOnChunkTransfer;
begin
 try
  //create stream
  MemStrm:=TMemoryStream.Create;
  //make sure no outputs are sent
  old:=FOnNewChunk;
  FOnNewChunk:=nil;
  //dump
  Dump(Address,Address+4,MemStrm);
  MemStrm.Position:=0;
  //store into output
  MemStrm.Read(buf,4);
  //restore output
  FOnNewChunk:=old;
  //swap to little endian
  Result:=BSwap(buf);
  //thank you!
  FreeAndNil(MemStrm);
 except on E:EUSBGeckoException do begin
  FreeAndNil(MemStrm);
  raise EUSBGeckoException.Create(E.Message);
  end;
 end;
end;

(* Poke 32 bit value
   address: address to poke
   value: value to write
   all little endian! *)
procedure TUSBGecko.Poke(address, value: Cardinal);
var PokeVal:Int64;
    TxSent,RXSent:DWord;
    ReturnValue,ResponseBack:Cardinal;
begin
 if not connected then
  raise EUSBGeckoException.Create('No conn');

 //value = send [address in big endian] [value in big endian]
 PokeVal:=Int64(BSwap(address))+Int64(Int64(BSwap(value)) shl 32);

 //Send poke
 returnvalue := gecko_writebyte(@cmd_pokemem, 1, @TxSent);
 if(returnvalue<>1) then
  raise EUSBGeckoException.Create('Error sending command');

 //write value
 returnvalue := gecko_writebyte(@PokeVal, 8, @TxSent);
 if(returnvalue <> 1) then
  begin
   raise EUSBGeckoException.Create('Error: Fatal USB Error writing memory location');
  end;

end;

(* Poke 16 bit value
   address: address to poke
   value: value to write
   no comments, works like poke (32 bit) just a different command! *)
procedure TUSBGecko.Poke16(address:Cardinal;extvalue:Word);
var PokeVal:Int64;
    value:Cardinal;
    TxSent,RXSent:DWord;
    ReturnValue,ResponseBack:Cardinal;
begin
 if not connected then
  raise EUSBGeckoException.Create('No conn');

 value:=extvalue;

 PokeVal:=Int64(BSwap(address))+Int64(Int64(BSwap(value)) shl 32);

 returnvalue := gecko_writebyte(@cmd_poke16, 1, @TxSent);
 if(returnvalue<>1) then
  raise EUSBGeckoException.Create('Error sending command');

 returnvalue := gecko_writebyte(@PokeVal, 8, @TxSent);
 if(returnvalue <> 1) then
  begin
   raise EUSBGeckoException.Create('Error: Fatal USB Error writing memory location');
  end;

end;

(* Poke 8 bit value
   address: address to poke
   value: value to write
   no comments, works like poke (32 bit) just a different command! *)
procedure TUSBGecko.Poke8(address:Cardinal;extvalue:Byte);
var PokeVal:Int64;
    value:Cardinal;
    TxSent,RXSent:DWord;
    ReturnValue,ResponseBack:Cardinal;
begin
 if not connected then
  raise EUSBGeckoException.Create('No conn');

 value:=extvalue;

 PokeVal:=Int64(BSwap(address))+Int64(Int64(BSwap(value)) shl 32);

 returnvalue := gecko_writebyte(@cmd_poke08, 1, @TxSent);
 if(returnvalue<>1) then
  raise EUSBGeckoException.Create('Error sending command');

 returnvalue := gecko_writebyte(@PokeVal, 8, @TxSent);
 if(returnvalue <> 1) then
  begin
   raise EUSBGeckoException.Create('Error: Fatal USB Error writing memory location');
  end;

end;

(* Read status
 Result: wsRunning=game is normally running
  wsPaused=game is paused
  wsBreakpoint=game is in bp mode
  wsLoader=we're currently in Gecko OS
  wsUnknown=probably crashed *)
function TUSBGecko.WiiStatus:TWiiStatus;
var returnvalue:Cardinal;
           stat:Byte;
  TxSent,RxSent:DWord;
begin
 //Reset
 InitGecko;

 //Send command
 returnvalue := gecko_writebyte(@cmd_status, 1, @TxSent);
 if(returnvalue<>1) then
  raise EUSBGeckoException.Create('Error sending command');

 //Read status
 returnvalue := gecko_readbyte(@stat, 1, @RxSent);
 if(returnvalue<>1) then
  raise EUSBGeckoException.Create('Error sending command');

 //Store status
 case stat of
  0: Result:=wsRunning;
  1: Result:=wsPaused;
  2: Result:=wsBreakpoint;
  3: Result:=wsLoader;
  else Result:=wsUnknown;
  end;
end;


(* JUst send a bare command - returns the command answer! *)
function TUSBGecko.RareCommand(id:byte): integer;
var TxSent:DWord;
begin
 Result:=gecko_writebyte(@id,1,@TxSent);
end;

(* Disconnect and connect - returns true if connection succeeded *)
function TUSBGecko.Reconnect:boolean;
begin
 //if not FConnected then
 // exit;
 Disconnect;
 try Result:=Connect;
 except
  Result:=false;
 end;
end;

(* Resume a paused game *)
procedure TUSBGecko.Resume;
var returnvalue:Cardinal;
       TxSent:DWord;
begin
 returnvalue := gecko_writebyte(@cmd_unfreeze, 1, @TxSent);
 if(returnvalue<>1) then
  raise EUSBGeckoException.Create('Error sending command');
end;

(* Step in bp mode *)
procedure TUSBGecko.Step;
begin
 if RareCommand(cmd_step)<>1 then
  raise EUSBGeckoException.Create('Error sending command');
end;

(* perform a XFB screenshot and store into a TBitmap object *)
procedure TUSBGecko.Screenshot(output: TBitmap);
var analyze:TMemoryStream;
    swidth,sheight:Word;


 procedure ProcessImage;

   type
    TYUVWord =    // A "Word" is always 2-bytes in all versions of Delphi
      record
        y:  BYTE;    // U if index even; V if index odd
        uv :  BYTE;    // Y for all array elements
      end;

    TRGBTripleArray  = array[0..1024-1] of TRGBTriple;
    pRGBTripleArray = ^TRGBTripleArray;


  // This may be "overkill".  Check upper and lower bounds to be safe.
  function FixValue(const x:  DOUBLE): Byte;
    var
      i:  integer;
  begin
    i := Round(x);
    if   i > 255  then Result := 255
    else
     if   i < 0
      then Result := 0
      else Result := Byte(i)
  end {FixValue};


  var
    BitmapRGB  :  TBitmap;
    BufferYUV  :  array[0..1024-1] of TYUVWord;
    i          :  integer;
    j          :  integer;
    Row        :  pRGBTripleArray;
    U          :  integer;
    V          :  integer;
    Y          :  integer;
    OutputBMP  :  TBitmap;


 begin
  try
   BitmapRGB := TBitmap.Create;
   try
    BitmapRGB.PixelFormat := pf24Bit;
    BitmapRGB.Width  := swidth;
    BitmapRGB.Height := sheight;
    for j := 0 to sheight-1 DO
     begin
      Analyze.Read(BufferYUV,  swidth*2);
      Row := BitmapRGB.Scanline[j];

      for i := 0 to swidth-1 DO
       begin
        if i mod 2 = 0 then  // Even
         begin
          Y := BufferYUV[i].Y;
          U := BufferYUV[i].UV;
          V := BufferYUV[i+1].UV     // get V from next word
         end
        else
         begin        // Odd
          Y := BufferYUV[i].Y;
          U := BufferYUV[i-1].UV;    // get U from last word
          V := BufferYUV[i].UV
         end;

        with Row[i] do
         begin
          // See "HELP! YUV-RGB conversion", 12/12/97 by
          // lweng@analogic.com for this formula:
          // http://www.deja.com/getdoc.xp?AN=297574991
          rgbtRed   := FixValue(1.164*(Y-16) + 1.596*(V-128));
          rgbtGreen := FixValue(1.164*(Y-16) - 0.392*(U-128) - 0.813*(V-128));
          rgbtBlue  := FixValue(1.164*(Y-16) + 2.017*(U-128))
         end
       end;
     end;
    Output.Assign(BitmapRGB);
   finally
    FreeAndNil(BitmapRGB);
   end;
  except end;
 end;

 procedure Realign(input:TBitmap);
 var
  output:TBitmap;
  y,x:integer;
  w,o:integer;
  scanlinein:PByteArray;
  scanlineout:PByteArray;
 begin
  w:=input.Width;
  input.PixelFormat:=pf24bit;
  output:=TBitmap.Create;
  output.PixelFormat:=pf24bit;
  output.Width:=w * 2;
  output.Height:=input.Height DIV 2;

  for y := 0 to output.Height-1 do
   begin
    scanlineout:=output.ScanLine[y];
    scanlinein:=input.ScanLine[y*2];
    for x := 0 to w*3-1 do
     scanlineout^[x]:=scanlinein^[x];
    o:=w*3;
    scanlinein:=input.ScanLine[y*2+1];
    for x := 0 to w*3-1 do
     scanlineout^[o+x]:=scanlinein^[x];
   end;

  input.Assign(output);

  FreeAndNil(output);
 end;

var viregs:array[0..127] of byte;
    soffset:Cardinal;
begin
 analyze:=TMemoryStream.Create;
 //Dump video registers
 Dump($CC002000,$CC002080,analyze);
 analyze.Position:=0;
 analyze.Read(viregs,128);
 analyze.Clear;

 //extract width, height and offset in memory
 swidth:=(viregs[$49] shl 3);
 sheight:=(((viregs[0] shl 5)or(viregs[1] shr 3)) and $07FE);
 soffset := ((viregs[$1D] shl 16) or (viregs[$1E] shl 8) or viregs[$1F]);
 if ((viregs[$1C] and $10) = $10) then soffset := soffset shl 5;
 soffset := soffset + $80000000;
 soffset := soffset - ((viregs[$1C] and $0F) shl 3);

 //dump video data
 Dump(soffset,soffset+sheight*swidth*2,analyze);
 analyze.Position:=0;
 //process the image
 ProcessImage;
 FreeAndNil(analyze);

 //in 480p realign lines!
 if sheight>600 then
  Realign(output);
end;

//Send an update to the output - used for dumps/uploads
procedure TUSBGecko.SendChunkUpdate(current, finl, bytestrans,
  memrange: integer;lastfail:boolean);
begin
 if(Assigned(FOnNewChunk)) then
  FOnNewChunk(current,finl,bytestrans,memrange,lastfail);
end;

//Send a GCFAIL command - in case the game halts this might help!
procedure TUSBGecko.SendFail;
var Txsent:DWord;
begin
 gecko_writebyte(@GCFAIL,1,@TxSent);
end;

//Send registers (they must be stored in a stream)
procedure TUSBGecko.SendRegs(sendStream:TStream);
var returnvalue:integer;
  TxSent,RXSent:DWord;
   responseback:Byte;
          retry:integer;

         buffer:string;

        retries:Byte;
        blength:Word;
            new:boolean;
            ver:Byte;
begin
 //Reset
 InitGecko;



 //FP registers CANNOT be sent
 //if new then blength:=$120
 //else
  blength:=$A0;

 //CHeck stream length
 if sendStream.Size<>blength then
  raise ERegStreamException.Create('Wrong stream size - file should be '+IntToStr(blength)+' bytes large!');

 SetLength(buffer,blength);
 sendStream.Position:=0;
 sendStream.Read(buffer[1],blength);

 //Send the registers
 returnvalue := gecko_writebyte(@cmd_sendregs, 1, @TxSent);
 if(returnvalue<>1) then
  raise EUSBGeckoException.Create('Error initiating transfer');

 retry:=0;
 while(retry<3) do
  begin
   returnvalue:=gecko_readbyte(@responseback,1,@RxSent);
   if returnvalue<>1 then
    raise EUSBGeckoException.Create('Return value '+IntToStr(returnvalue)+' while asking for status!');
   if responseback=GCACK then
    break;
   if retry=2 then
    raise EUSBGeckoException.Create('Invalid status reply - reply is not GCACK - last reply was: '+IntToHex(responseback,2));
   Inc(retry);
  end;

 retry:=0;
 while(retry<3) do
  begin
    SendChunkUpdate(0,0,0,0,retry<>0);
    returnvalue:=gecko_writebyte(@buffer[1],blength,@TxSent);
    if returnvalue=2 then
     begin
       Inc(retry);
       if retry=3 then
        begin
          //Give up too many retries!
          gecko_writebyte(@GCFAIL,1,@TxSent);
          raise EUSBGeckoException.Create('Too many retries while trying to send the registers!');
        end;
       gecko_writebyte(@GCRETRY,1,@TxSent);
       continue;
     end
    else if returnvalue=0 then
     begin
       //Major fail, give up
       gecko_writebyte(@GCFAIL,1,@TxSent);
       raise EUSBGeckoException.Create('Gecko error - please check your USB Gecko connection!');
     end;
    retry:=3;
    //gecko_writebyte(@GCACK,1,@TxSent);
    break;
  end;
end;

(*pretty much the opposite of dump, I only explain headers!
  address: start address
  length: data length
  sendStream: STream to read data from *)
procedure TUSBGecko.Upload(address, length: Cardinal; sendStream: TStream);
var GeckoMemRange:Int64;
    TxSent,RxSent:DWord;
      returnvalue:integer;
     responseback:Byte;

            chunk:Integer;
       fullchunks:Integer;
        allchunks:Integer;
        lastchunk:Integer;

            retry:integer;
           buffer:string;
             // jff:boolean;
begin
 InitGecko;

 if length>sendStream.Size-sendStream.Position then
  length:=sendStream.Size-sendStream.Position;

 fullchunks:=length DIV uplpacketsize;
  lastchunk:=length MOD uplpacketsize;

 allchunks:=fullchunks;
 if(lastchunk>0) then
  Inc(allchunks);

 //jff:=lastchunk=0;
 //if lastchunk=0 then
 // FixDumps;

 GeckoMemRange:=BSwap64(Int64(address) shl 32+length);

 returnvalue:=gecko_writebyte(@cmd_upload,1,@TxSent);
 if returnvalue<>1 then
  raise EUSBGeckoException.Create('Unable to send upload command to USBGecko!');

 retry:=0;
 while(retry<3) do
  begin
   returnvalue:=gecko_readbyte(@responseback,1,@RxSent);
   if returnvalue<>1 then
    raise EUSBGeckoException.Create('Return value '+IntToStr(returnvalue)+' while asking for status!');
   if responseback=GCACK then
    break;
   if retry=2 then
    raise EUSBGeckoException.Create('Invalid status reply - reply is not GCACK - last reply was: '+IntToHex(responseback,2));
   Inc(retry);
  end;


 returnvalue:=gecko_writebyte(@GeckoMemRange,8,@TxSent);
 if returnvalue<>1 then
  raise EUSBGeckoException.Create('Failed sending range!');

 chunk:=0;


 retry:=0;
 while(chunk < fullchunks) do
  begin
    SendChunkUpdate(chunk,allchunks,chunk*uplpacketsize,length,retry<>0);
    SetLength(buffer,uplpacketsize);
    sendStream.Read(buffer[1],uplpacketsize);
    returnvalue:=gecko_writebyte(@buffer[1],uplpacketsize,@RxSent);
    if returnvalue=2 then
     begin
       Inc(retry);
       if retry=3 then
        begin
          //Give up too many retries!
          gecko_writebyte(@GCFAIL,1,@TxSent);
          raise EUSBGeckoException.Create('Too many retries while trying to send a package!');
        end;
       gecko_writebyte(@GCRETRY,1,@TxSent);
       continue;
     end
    else if returnvalue=0 then
     begin
       //Major fail, give up
       gecko_writebyte(@GCFAIL,1,@TxSent);
       raise EUSBGeckoException.Create('Gecko error - please check your USB Gecko connection!');
     end;

    returnvalue:=gecko_readbyte(@responseback,1,@RxSent);
    if returnvalue=0 then
     begin
       //Major fail, give up
       gecko_writebyte(@GCFAIL,1,@TxSent);
       raise EUSBGeckoException.Create('Gecko error - please check your USB Gecko connection!');
     end
    else if (returnvalue=2) or (responseback<>$AA) then
     begin
      Inc(retry);
      if retry=3 then
       begin
         //Give up too many retries!
         gecko_writebyte(@GCFAIL,1,@TxSent);
         raise EUSBGeckoException.Create('Too many retries while trying to send a package!');
       end;
      gecko_writebyte(@GCRETRY,1,@TxSent);
      continue;
     end;

    retry:=0;
    Inc(chunk);
  end;

 while(lastchunk>0) do
  begin
    SendChunkUpdate(chunk,allchunks,fullchunks*uplpacketsize,length,retry<>0);
    SetLength(buffer,lastchunk);
    sendStream.Read(buffer[1],lastchunk);
    returnvalue:=gecko_writebyte(@buffer[1],lastchunk,@RxSent);
    if returnvalue=2 then
     begin
       Inc(retry);
       if retry=3 then
        begin
          //Give up too many retries!
          gecko_writebyte(@GCFAIL,1,@TxSent);
          raise EUSBGeckoException.Create('Too many retries while trying to send a package!');
        end;
       gecko_writebyte(@GCRETRY,1,@TxSent);
       continue;
     end
    else if returnvalue=0 then
     begin
       //Major fail, give up
       gecko_writebyte(@GCFAIL,1,@TxSent);
       raise EUSBGeckoException.Create('Gecko error - please check your USB Gecko connection!');
     end;
    retry:=0;
    lastchunk:=0;
    //gecko_writebyte(@GCACK,1,@TxSent);
  end;
 SetLength(buffer,0);
end;

//For sending cheats - it should insert a start
//00D0C0DE 00D0C0DE
//so that the code handler knows things are starting 
procedure InsertStartCode(strm:TStream);
var memStrm:TMemoryStream;
     insert:Int64;
begin
 memStrm:=TMemoryStream.Create;
 insert:=BSwap64($00D0C0DE00D0C0DE);
 memStrm.Write(insert,8);
 strm.Position:=0;
 memStrm.CopyFrom(strm,strm.Size);
 strm.Position:=0;
 memStrm.Position:=0;
 strm.CopyFrom(memStrm,memStrm.Size);
 FreeAndNil(memStrm);
end;

//Kinda like upload, just it sends a stream containing cheat data
procedure TUSBGecko.SendCheats(sendStream: TStream);
var GeckoMemRange:Cardinal;
    TxSent,RxSent:DWord;
      returnvalue:integer;
     responseback:Byte;
             data:Byte;
             send:Int64;
           length:Cardinal;

            chunk:Integer;
       fullchunks:Integer;
        allchunks:Integer;
        lastchunk:Integer;

            retry:integer;
           buffer:string;
begin
 length:=sendStream.Size;

 if length MOD 8<>0 then
  raise ECheatStreamException.Create('Cheat list must be a multiple of 8 bytes in length!');

 InitGecko;

 data:=0;
 sendStream.Position:=sendStream.Size-8;
 sendStream.read(data,1);
 if data<$F0 then
  begin
   send:=BSwap64($F000000000000000);
   sendStream.Position:=sendStream.Size;
   sendStream.Write(send,8);
  end;

 sendStream.Position:=0;
 sendStream.Read(send,8);
 if(send<>BSwap64($00D0C0DE00D0C0DE)) then
  begin
   InsertStartCode(sendStream);
  end;

 sendStream.Position:=0;

 length:=sendStream.Size;

 if length>$1000 then
  raise ECheatStreamException.Create('Cheat list is too long, mustn''t exceed 770 bytes!');

 fullchunks:=length DIV uplpacketsize;
  lastchunk:=length MOD uplpacketsize;

 allchunks:=fullchunks;
 if(lastchunk>0) then
  Inc(allchunks);
 
 GeckoMemRange:=BSwap(length);

 returnvalue:=gecko_writebyte(@cmd_sendcheats,1,@TxSent);
 if returnvalue<>1 then
  raise EUSBGeckoException.Create('Unable to send upload command to USBGecko!');

 retry:=0;
 while(retry<3) do
  begin
   returnvalue:=gecko_readbyte(@responseback,1,@RxSent);
   if returnvalue<>1 then
    raise EUSBGeckoException.Create('Return value '+IntToStr(returnvalue)+' while asking for status!');
   if responseback=GCACK then
    break;
   if retry=2 then
    raise EUSBGeckoException.Create('Invalid status reply - reply is not GCACK - last reply was: '+IntToHex(responseback,2));
   Inc(retry);
  end;


 returnvalue:=gecko_writebyte(@GeckoMemRange,4,@TxSent);
 if returnvalue<>1 then
  raise EUSBGeckoException.Create('Failed sending range!');

 chunk:=0;


 retry:=0;
 while(chunk < fullchunks) do
  begin
    SendChunkUpdate(chunk,allchunks,chunk*uplpacketsize,length,retry<>0);
    SetLength(buffer,uplpacketsize);
    sendStream.Read(buffer[1],uplpacketsize);
    returnvalue:=gecko_writebyte(@buffer[1],uplpacketsize,@RxSent);
    if returnvalue=2 then
     begin
       Inc(retry);
       if retry=3 then
        begin
          //Give up too many retries!
          gecko_writebyte(@GCFAIL,1,@TxSent);
          raise EUSBGeckoException.Create('Too many retries while trying to send a package!');
        end;
       gecko_writebyte(@GCRETRY,1,@TxSent);
       continue;
     end
    else if returnvalue=0 then
     begin
       //Major fail, give up
       gecko_writebyte(@GCFAIL,1,@TxSent);
       raise EUSBGeckoException.Create('Gecko error - please check your USB Gecko connection!');
     end;

    returnvalue:=gecko_readbyte(@responseback,1,@RxSent);
    if returnvalue=0 then
     begin
       //Major fail, give up
       gecko_writebyte(@GCFAIL,1,@TxSent);
       raise EUSBGeckoException.Create('Gecko error - please check your USB Gecko connection!');
     end
    else if (returnvalue=2) or (responseback<>$AA) then
     begin
      Inc(retry);
      if retry=3 then
       begin
         //Give up too many retries!
         gecko_writebyte(@GCFAIL,1,@TxSent);
         raise EUSBGeckoException.Create('Too many retries while trying to send a package!');
       end;
      gecko_writebyte(@GCRETRY,1,@TxSent);
      continue;
     end;

    retry:=0;
    Inc(chunk);
  end;

 while(lastchunk>0) do
  begin
    SendChunkUpdate(chunk,allchunks,fullchunks*uplpacketsize,length,retry<>0);
    SetLength(buffer,lastchunk);
    sendStream.Read(buffer[1],lastchunk);
    returnvalue:=gecko_writebyte(@buffer[1],lastchunk,@RxSent);
    if returnvalue=2 then
     begin
       Inc(retry);
       if retry=3 then
        begin
          //Give up too many retries!
          gecko_writebyte(@GCFAIL,1,@TxSent);
          raise EUSBGeckoException.Create('Too many retries while trying to send a package!');
        end;
       gecko_writebyte(@GCRETRY,1,@TxSent);
       continue;
     end
    else if returnvalue=0 then
     begin
       //Major fail, give up
       gecko_writebyte(@GCFAIL,1,@TxSent);
       raise EUSBGeckoException.Create('Gecko error - please check your USB Gecko connection!');
     end;
    retry:=0;
    lastchunk:=0;
    //gecko_writebyte(@GCACK,1,@TxSent);
  end;
 SetLength(buffer,0);
end;

end.
